/* Copyright (C) 2002 GFRN systems

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   The latest version of this program may be found at
   http://CQiNet.sourceforge.net

   $Log: ports.h,v $
   Revision 1.3  2007/12/07 16:57:23  wb6ymh
   1. Moved global ADPCM variables into CPort class.
   2. Added prototypes for CancelToneGen, SourcesReady, and GetClientAudio.
   3. Added TimeoutToneGen, CtoneResetsTimeout, and b8BitFile variables.

   Revision 1.2  2007/12/01 06:03:49  wb6ymh
   1. Added include for new ToneGen.h.
   2. Deleted GenTestSweep.
   3. Added class members and variables to implement courtesy tone, hangtime
      and transmitter timeout functions.

   Revision 1.1  2007/11/26 15:19:55  wb6ymh
   Inital import - header for CPort class.

*/

#ifndef _PORTS_H_
#define _PORTS_H_

#ifdef USB_SUPPORT
#include <usb.h>
#endif
#include "ToneGen.h"
#include "sf.h"
#include "adpcm.h"

#define MAX_PORTS 32

typedef struct PlayPCM_TAG {
   struct PlayPCM_TAG *Link;  // NB: must be first
   int   Flags;
      #define PLAY_FLAG_ID       0x1   // voice ID
      #define PLAY_FLAG_TTS      0x2   // FileName = Text for TTS system
      #define PLAY_FLAG_TTS_OUT  0x4   // audio rendered by TTS system
      
   char  *FileName;
} PlayPCM;

typedef struct DTMF_Command_Tag {
   struct DTMF_Command_Tag *Link;   // NB: must be first
   char  *DTMF_String;
   char  *Command;
} DTMF_Command;

class CPort 
{
public:
// public functions   
   CPort();
// entry points from C code
   int EndPointInit();
   int WriteAudioOut(ClientInfo *p);
   int Audio_Handler(ClientInfo *p);
   void GetAudioIn(ClientInfo *p);
   void AudioCleanup(ClientInfo *p);
   int InitToneBuf(struct config_entry *p, char *Arg);
   int DeviceInHandler(ClientInfo *p);
   void SaveDTMF(char Digit);
   void DecodeDTMF(char *Command);
   void CmdForceID();
   void SayPhrase(char *Phrase);
   void SayCall(char *Call);
   int Shutup();
   int LoadCommandFile(struct config_entry *p, char *Arg);
   void AddToneGen(CToneGen *pGenerator);
   void CancelToneGen(int id);

// Public variables

// Bit map of which ports have Rx audio ready for them
   int RxAudioReady;

// Bit Map of rf ports this port is listening to
   int RxMap;

   int MyBit;

// variables accessed by C code
   int  PortNum;

// Config variables
   int VoxThreshold;
   int VoxHoldTime;
   int VoxTripDelay;
   int DtmfMethod;
   int TxKeyMethod;
   int RxCosMethod;
   int SerialBaudRate;
   int DTMFPollTime;
   int InvertPTT;
   int InvertCOS;
   int CWToneLevel;
   int CWSpeed;
   int CWAutoJam;
   int IdInterval;
   int VoiceIDIdleTime;
   int HalfDuplexRF;
   int HalfDuplex;
   int PCMRate;
   int CTCSSLevel;
   int Twist;
   int ReverseTwist;
   int CtcssDecoderSamples;
   int DecodeCTCSS;
   int MuteDTMF;
   int bMonitor;
   int COS_KeyCode;
   int AudioIn8Bit;
   int CWToneFreq;
   int bDiscFeed;
   int bUngatedRx;
   int PTT_Bit;
   int COS_Bit;
   int Ctone;
   int CtoneMap;
   int CtoneDelay;
   int HangTime;
   int TxTimeout;
   int TimeoutTone;
   int CtoneResetsTimeout;

   float EchoCTCSSFreq;

   char *SerialDevName;
   char *CWId;
   char *VoiceID;
   char *AudioDevice;
   char *AudioDongleSN;
   char *AudioDongleAdr;
   char *PortName;

private:
   struct timeval LastCOSPoll;
   struct timeval LastSpoke;
   struct timeval LastDTMFDigit;
   struct timeval LastVoxTrip;
   struct timeval LastDTMFPoll;
   struct timeval CtoneTimer;
   struct timeval VoipCtoneTimer;
   struct timeval HangTimer;
   time_t LastID;
   time_t LastActivity;
   int bTxKeyed;
   int bTalking;     // receiving signal from RF user
   int bTxIsKeyed;
   int bDevInCOS;
   int bTossRxIn;
   int g_bCosActive;
   void *pDtmfRxState;
   FILE *Playfp;
   PlayPCM *pPlayHead;
   PlayPCM *pPlayTail;

   #undef MAX_DTMF_DIGITS
   #define MAX_DTMF_DIGITS    16
   char  DTMF_Buf[MAX_DTMF_DIGITS+1];
   int   DTMF_Digits;
   int MaxMixedLevel;
   int LastMixGain;
   double CTCSSAngle;
   char *CWChar;
   int ElementCount;
   int bCwToneOn;
   int bElementSpace;
   int RampCount;
   char *Element;
   int AudioInAvailable;
   char *AudioInBuf8;
   int16 *AudioInBuf;
   ClientInfo *pAudioC;
   int PlayAvail;
   DTMF_Command *pDTMFCommands;
   CPort *pSerialSlave;
   CPort *pParallelSlave;
   int SerialFd;
   int ModemCtrl;

   CToneGen *ToneGenerators;
   CToneGen *TimeoutToneGen;
   time_t TxKeyTime;
   int bTimeOut;
   int DelayedCtoneMap;
   int b8BitFile;
   struct AdpcmTAG {
      union {
         soundbuf SfHdr;
         rtp_hdr_t RtpHdr;
      } u;
      char AdpcmStateBuf[4];
      char Buf[ADPCM_PACKET_DATA_BYTES];
      char AdpcmStateBuf1[3];
   } GCC_PACKED Adpcm;
   
   int ADPCM_Bytes; // number of bytes of compressed ADPCM data in buffer
   
   struct adpcm_state TxADPCMStart;
   struct adpcm_state TxADPCM;

#ifdef USB_SUPPORT
   usb_dev_handle *udev;
   int UsbDevType;
#endif
// Kludge... fix me!
   int bAudioOutCalled;

   void KeyTx(int bKey);
   void ID();
   void FillAudioBuf();
   void Vox();
   void ProcessAudioIn(ClientInfo *p);
   void PollCOS(ClientInfo *p);
   void PollDTMF(ClientInfo *p);
   int SerialInit();
   void FillCTCSSBuf();
   int InitDeviceInput();
   int UsbInit(void);
   void StartTalking(int bTalk);
   void StartNextAnnoucement();
   void PlayAnnouncementFile(char *FileName,int Flags);
   int GetAnnounceAudio();
   void FreeAnnoucement();
   void MixAudio(int16 *In);
   void ScaleMixedAudio();
   void GenCTCSS(int16 *CTCSSBuf);
   void GenCW(int16 *CwBuf);
   void SendCWMsg(char *Msg);
   void AdpcmTx(int SfAdpcmCount,int RtpAdpcmCount);
   void GsmTx(int SfGsmCount,int RtpGsmCount,int EchoLinkCount);
   void UsbKeyRadio(int bKey);
   int UsbPollCOS();
   int ActivePorts();
   int SourcesReady(ConfClient **pTalker);
   int FindSerialMaster();
   void SetTimeout(struct timeval *pTimer,int Timeout);
   void SendCtone();
   void GenVoipCtone();
   int VoipAudioAvailable(ConfClient *pCC);
   int PlayStoredSound(int ID);
   int GetClientAudio(ConfClient *pCC);
};

extern CPort *Ports[MAX_PORTS];

// bit map of ports that have active receive signals
extern int CosMap;
#endif   // _PORTS_H_

